from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
chrome_options = webdriver.ChromeOptions()
chrome_options.add_argument('--headless')
chrome_options.add_argument('--disable-dev-shm-usage')
import pandas as pd
import time
import csv

from yt_requests import yt_search, yt_comments

def comments_to_csv(query, API_KEY, publishedBefore, publishedAfter, maxResults=49, driver_path="C:/WebDriver/bin/chromedriver.exe", csv_path="./youtube_comments.csv", useAPI=True):
    """
    Search YouTube video and comment info based on query search results and write data to a csv file.
    If `useAPI` is set to False, `youcos` will scrape the comments for each video using Selenium.
    
    Parameters
    ----------
    query : string
        The query to search for on YouTube
    API_KEY : string
        The API key to authenticate requests to YouTube Data API v3
    maxResults : int, optional
        The maximum number of videos to scrape
    driver_path : string, optional
        The browser path for Selenium (the default is "C:/WebDriver/bin/chromedriver.exe", which is the typical location for Chrome drivers)
    csv_path : string, optional
        The file path to save csv file (the default is "../comments.csv", which saves the file in the directory above the current one)
    useAPI : boolean, optional
        If False, `youcos` scrapes comments for each video using Selenium (the default is True, which makes `youcos` use YouTube v3 API to request comments)
    """

    
    video_list = request_videos(query, API_KEY,  publishedBefore, publishedAfter, maxResults=maxResults)
    
    if (useAPI):
        request_comments(video_list, API_KEY, csv_path)
    else:
        scrape_comments(video_list, driver_path, csv_path)

def request_videos(query, API_KEY, publishedBefore, publishedAfter, maxResults=49, driver_path="C:/WebDriver/bin/chromedriver.exe"):
    """
    Search YouTube videos based on the given query and return a list of dictionaries containing url, title, and search query.

    Parameters
    ----------
    query : string
        The query to search for on YouTube
    API_KEY : string
        The API key to authenticate requests to YouTube Data API v3
    driver_path : string, optional
        The browser path for Selenium (the default is "C:/WebDriver/bin/chromedriver.exe", which is the typical location for Chrome drivers)
    Returns
    ----------
    video_list : list of dict
        The list of collected video data, each dictionary with the video's url, title, and search query 
    Notes
    ----------
    For more info on YouTube v3 API, please visit https://developers.google.com/youtube/v3
    """
    
    video_list = yt_search(query, API_KEY, publishedBefore, publishedAfter, maxResults)
    
    # Check if there are no video results
    if not video_list:
        return
    
    for video in video_list:
        video['query'] = query
    return video_list

def request_comments(video_list, API_KEY, csv_path="../comments.csv", as_df=False):
    """
    Request comment data using the YouTube v3 API, then write video and comment data to a csv file or return as a Pandas DataFrame if
    `as_df` is `True`

    Parameters
    ----------
    video_list : list of dict
        The list of videos to fetch comments
    API_KEY : string
        The API key to authenticate requests to YouTube Data API v3
    csv_path : string, optional
        The location to save the csv file containing comments data
    as_df : boolean, optional
        If True, return data as a Pandas Dataframe (default is False).
    """
    columns = ['query', 'url', 'title', 'upload_date', 'channel', 'views', 'likes', 'dislikes', 'comment_count', 'comment_text', 'comment_author', 'comment_date', 'comment_likes']
    df = pd.DataFrame(columns=columns)
    
    # If video list is empty, return empty
       
    for video in video_list:
        
        # Grab all comments for video
        comments = yt_comments(video['id'], API_KEY)
        
        # Skip video if comments are disabled
        if not comments:
            continue
        
        for comment in comments:
            youtube_dict = {}
            
            # Write scraped data to csv file
            youtube_dict['query'] = video['query']
            youtube_dict['url'] = "https://www.youtube.com/watch?v=" + video['id']
            youtube_dict['title'] = video['title']
            youtube_dict['upload_date'] = video['date']
            youtube_dict['channel'] = video['channel']
            youtube_dict['views'] = video['views']
            youtube_dict['likes'] = video['likes']
            youtube_dict['dislikes'] = video['dislikes']
            youtube_dict['comment_count'] = video['comment_count']
            youtube_dict['comment_text'] = comment['text']
            youtube_dict['comment_author'] = comment['author']
            youtube_dict['comment_date'] = comment['date']
            youtube_dict['comment_likes'] = comment['likes']
            df = df.append(youtube_dict, ignore_index=True)
    
    if as_df:
        return df
    
    df.to_csv(csv_path, encoding="UTF-8", index=False)
    return
    
def scrape_comments(video_list, driver_path="C:/WebDriver/bin/chromedriver.exe", csv_path="../comments.csv"):
    """
    Scrape YouTube video and comment info using Selenium, then write data to a csv file.

    Parameters
    ----------
    video_list : list of dict
        The list of videos to scrape
    driver_path : string, optional
        The browser path for Selenium (the default is "C:/WebDriver/bin/chromedriver.exe", which is the typical location for Chrome drivers)
    csv_path : string, optional
        The location to save the csv file containing comments data
    """
    
    csv_file = open(csv_path,'w', encoding="UTF-8", newline="")
    writer = csv.writer(csv_file)    
    
    writer.writerow(['query', 'url', 'title', 'upload_date', 'channel', 'no_of_views', 'likes', 'dislikes', 'comment', 'author', 'comment_date', 'no_of_replies','upvotes'])    
    driver = webdriver.Chrome(executable_path=driver_path)

    for video in video_list:
        
        url = video['url']
        title = video['title']
        upload_date = video['date']
        query = video['query']
        
        # Scrape basic video data
        print("=" * 40)
        print("video title : ", title)
        driver.get(url)
        v_channel = WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR,"div#upload-info yt-formatted-string"))).text
        print("channel : ",v_channel)    
        v_views = WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR,"div#count span.view-count"))).text
        print("no. of views : ",v_views)
        v_timeUploaded = WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR,"div#date yt-formatted-string"))).text
        print("time uploaded : ",v_timeUploaded)
        w = WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR,"div#top-level-buttons yt-formatted-string")))
        w = driver.find_elements_by_css_selector("div#top-level-buttons yt-formatted-string")
        v_likes = w[0].text
        v_dislikes = w[1].text
        print("video has ", v_likes, "likes and ", v_dislikes, " dislikes")
        
        youtube_dict ={}
    
        print("+" * 40)
        print("Scraping child links ")
        
        # Load comments section
        driver.execute_script('window.scrollTo(0,390);')
        time.sleep(2)
        
        try:
            # Sort by top comments
            print("sorting by top comments")
            sort= WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR,"div#icon-label")))
            sort.click()
            topcomments =driver.find_element_by_xpath("""//*[@id="menu"]/a[1]/paper-item/paper-item-body/div[1]""")
            topcomments.click()
            
            # Loads more comments
            for i in range(0,5):
                driver.execute_script("window.scrollTo(0,Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,document.documentElement.clientHeight))")
                print("scrolling to load more comments")
                time.sleep(4)
    
            # Count total number of comments and set index to number of comments if less than 50 otherwise set as 50. 
            totalcomments= len(driver.find_elements_by_xpath("""//*[@id="content-text"]"""))
    
            if totalcomments < 100:
                index= totalcomments
            else:
                index= 100 
            
            # Loop through each comment and scrape info
            print("scraping through comments")
            ccount = 0
            while ccount < index: 
                try:
                    comment = driver.find_elements_by_xpath('//*[@id="content-text"]')[ccount].text
                except:
                    comment = ""
                try:
                    authors = driver.find_elements_by_xpath('//a[@id="author-text"]/span')[ccount].text
                except:
                    authors = ""
                try:
                    comment_date = driver.find_elements_by_xpath('//*[@id="published-time-text"]/a')[ccount].text
                except:
                    comment_date = ""
                try:
                    replies = driver.find_elements_by_xpath('//*[@id="more-text"]')[ccount].text                    
                    if replies =="View reply":
                        replies= 1
                    else:
                        replies =replies.replace("View ","")
                        replies =replies.replace(" replies","")
                except:
                    replies = ""
                try:
                    upvotes = str(driver.find_elements_by_xpath('//*[@id="vote-count-middle"]')[ccount].text)
                except:
                    upvotes = ""
                
                              
                # Write scraped data to csv file
                youtube_dict['query'] = query
                youtube_dict['url'] = url
                youtube_dict['title'] = title
                youtube_dict['upload_date'] = upload_date
                youtube_dict['channel'] = v_channel
                youtube_dict['no_of_views'] = v_views
                youtube_dict['likes'] = v_likes
                youtube_dict['dislikes'] = v_dislikes
                youtube_dict['comment'] = comment
                youtube_dict['author'] = authors
                youtube_dict['comment_date'] = comment_date
                youtube_dict['no_of_replies'] = replies
                youtube_dict['upvotes'] = upvotes
                writer.writerow(youtube_dict.values())
                
                ccount = ccount + 1
        
        # If video errors out, move onto the next one
        except TimeoutException as e:
            print(title, "  errored out: ",str(e))
            print("moving onto next video")
            


from dotenv import load_dotenv
import os
import time
from scrape_youtube import comments_to_csv, request_videos, request_comments, scrape_comments

def main():
    load_dotenv()
    
    # Authenticate wit h YouTube API
    KEY = os.environ.get('API_KEY')
    
    
    #q = "coronavirus|covid%7C19|pandemic|vaccine"
    q="bts"
    month=str(7)
    year=str(2020)
    for day in range(1,31):
        driver_path = "C:/WebDriver/bin/chromedriver.exe"
        #csv_path = "./data/covid_2020_dec/" + 'covid' + "_2020_12_" + str(i) + ".csv"
        csv_path = "./data/" + q + "_" + year + "_" + month + "/" + q + "_" + year + "_" + month + "_" + str(day) + ".csv"
        publishedAfter = year + "-" + month + "-" + str(day) + "T00:00:00.00-05:00"  # March 2nd, 9:30AM
        publishedBefore = year + "-" + month + "-" + str(day+1) + "T00:00:00.00-05:00"
        print("scraping ", q)
        start = time.time()
        comments_to_csv(query=q, API_KEY=KEY, publishedBefore=publishedBefore, publishedAfter=publishedAfter,csv_path=csv_path)
        end = time.time()
        print("time elapsed: ", end - start)
        
if __name__ == "__main__":
    main()


from googleapiclient.discovery import build

def yt_search(query, API_KEY, publishedBefore, publishedAfter, maxResults=20):
    """
    Request YouTube video search results for a particular query and return parsed data

    Parameters
    ----------
    query : string
        The query to search for on YouTube
    API_KEY : string
        The API key to authenticate requests to YouTube Data API v3
    maxResults : int, optional
        The maximum number of videos to scrape
    Returns
    ----------
    videos : list of dict or None
        The list of collected video data, each dictionary with the video's url and title, or None if
        no YouTube search results exist
    Notes
    ----------
    For more info on YouTube v3 API, please visit https://developers.google.com/youtube/v3
    """
    
    if not API_KEY:
        return '"Please provide "YouTube API Key"'
    
    youtube = build('youtube', 'v3', developerKey=API_KEY)
    
    # Request search results
    try:
        search_response = youtube.search().list(
            q=query,
            maxResults=maxResults,
            part='id',
            type='video',
            order='viewCount',
            relevanceLanguage='en',
            publishedBefore=publishedBefore,
            publishedAfter=publishedAfter
        ).execute()
    except:
        raise

    search_items = search_response['items']
    
    videos = None
    
    if not search_items:
        return videos
    
    id_str = ""
    
    # Request video data
    for video in search_items:
        id_str = id_str + video["id"]["videoId"] + ","
    
    videos_response = youtube.videos().list(
        part="snippet,contentDetails,statistics",
        id=id_str
    ).execute()
    video_items = videos_response['items']
    
    videos = list(map(parse_videos, video_items))
    
    return videos

def yt_comments(videoId, API_KEY):
    """
    Request YouTube comments for a particular video and return parsed data

    Parameters
    ----------
    videoId : string
        The ID of video to fetch comments
    API_KEY : string
        The API key to authenticate requests to YouTube Data API v3
    Returns
    ----------
    comments : list of dict or None
        The list of collected comments data, each dictionary with the comment's text, author, publish date, and number of likes, or None if
        no YouTube search results exist
    Notes
    ----------
    For more info on YouTube v3 API, please visit https://developers.google.com/youtube/v3
    """
    
    if not API_KEY:
        return '"Please provide "YouTube API Key"'
    
    youtube = build('youtube', 'v3', developerKey=API_KEY)
    
    try:
        # Grab comments for videoId
        comments_response = youtube.commentThreads().list(
            part='id,snippet',
            videoId=videoId,
            maxResults=100,
            order="relevance"
    
        ).execute()
    
    except:
        return
    
    items = comments_response['items']
    comments = None
    
    if not items:
        return comments
    
    comments = list(map(parse_comments, items))
    
    return comments
    
def parse_videos(video):
    """
    Parse data from the video JSON structure
    
    Parameters
    ----------
    video : JSON object of strings
        The video search result requested from YouTube API v3
    Returns
    ----------
    video_dict : dictionary of strings
        The dictionary contianing video title, published date, id, views, likes, dislikes, and no. of comments
    """
    
    
    title = video['snippet']['title']
    date = video['snippet']['publishedAt']
    channel = video['snippet']['channelTitle']
    ID = video['id']
    stats = video['statistics']
    views = stats['viewCount']
    try:
        likes = stats['likeCount']
        dislikes = stats['dislikeCount']
    except:
        likes = 0
        dislikes = 0
    try:
        commentCount = stats['commentCount']
    except:
        commentCount = 0
    
    video_dict = {
        'title': title,
        'date': date,
        'channel': channel,
        'id': ID,
        'views': views,
        'likes': likes,
        'dislikes': dislikes,
        'comment_count': commentCount
    }
    
    return video_dict
    
def parse_comments(commentThread):
    """
    Parse data from the comment JSON structure
    
    Parameters
    ----------
    comment : JSON object of strings
        The comment resource requested from YouTube API v3
    Returns
    ----------
    comment_dict : dictionary of strings
        The dictionary contianing comment text, author, date, no. of likes
    """
    comment = commentThread['snippet']['topLevelComment']
    
    snippet = comment['snippet']
    text = snippet['textOriginal'].replace("\n", " ").replace("\t", " ").replace("\r", " ")
    author = snippet['authorDisplayName']
    date = snippet['publishedAt']
    likes = snippet['likeCount']
    
    comment_dict = {
        'text' : text,
        'author' : author,
        'date' : date,
        'likes' : likes
    }
    
    return comment_dict


"""Setup for the youcos package."""

import setuptools

with open('README.md') as f:
    README = f.read()

setuptools.setup(
    author="Seunggun Lee",
    author_email="seungguini@gmail.com",
    name='youcos',
    license="MIT",
    description='youcos is a simple Python package for collecting YouTube videos and comments',
    version='v0.0.6',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/seungguini/youcos',
    packages=setuptools.find_packages(),
    python_requires=">=3.5",
    install_requires=['selenium', 'google-api-python-client'],
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
    ],
)
